@@ -1,11 +1,10 @@
-
-__version__ = "3.1.0"
-__author__ = 'Michael Davis'
-__license__ = 'MIT'
-__copyright__ = 'Copyright 2016 Michael Davis'
+__version__ = "3.4.0"
+__author__ = "Michael Davis"
+__license__ = "MIT"
+__copyright__ = "Copyright 2016 Michael Davis"
 
 
+from .exceptions import ExpiredSignatureError  # noqa: F401
 from .exceptions import JOSEError  # noqa: F401
 from .exceptions import JWSError  # noqa: F401
-from .exceptions import ExpiredSignatureError  # noqa: F401
 from .exceptions import JWTError  # noqa: F401
